/////////////////////////////////////////////////////////////////////////////
// @doc ButtonHandler
// @module ButtonHandler.Java | Implementation of the <c ButtonHandler> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: ButtonHandler.java,v 1.3 2002/08/06 21:01:30 assefa Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: ButtonHandler.java,v $
// Revision 1.3  2002/08/06 21:01:30  assefa
// When a click happens on a button, it is set to press wheter it was already
// pressed or not.
//
// Revision 1.2  2000/09/19 21:59:20  assefa
// Add support for CURSOR_TYPE parameter.
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;

/**
 * This object is used to manage the mouse event associate to the ButtonCtrl.
 * It will send the command associate to the attached button.  
 *
 * @author  Guy Parent
 * 
 * @see ButtonCtrl
 * @see MouseAdapter
 */
class ButtonHandler extends MouseAdapter 
{
    /** Keep the attached button.    */
    ButtonCtrl  m_button;
    /** If it <b>true<b/> it will unpress the button after the action is perform. Otherwise
     * the button will be stay in pressed state. 
     */
    boolean m_bAutoRelease;
        
    /**
     * Constructor. 
     * @param button Button which listen the events.
     * @param bAutomaticRelease If <b>true</b>, the button will automaticaly
     *               unpress after sending the command.
     */
    public ButtonHandler( ButtonCtrl button, boolean bAutomaticRelease  )
    {
        m_button = button;  
        m_bAutoRelease = bAutomaticRelease;
    }

    /**
     * Anwser to the mouse press event. It will send the command to the listener attached
     * to the button.
     * @param e Mouse event object
     * @see MouseEvent
     */
    public void mousePressed(MouseEvent e) 
    { 
        // Set the button State.
        //When a click happens on a button, it is set to press wheter it was already
        //pressed or not. 
        if(true)//m_button.isPressed() == false)
        {
            m_button.setPressed(true); 
            m_button.setCursorType();
        }
        else
        {
            m_button.setPressed(  false ); 
            m_button.setCursorToDefault();
        }
        // Send the command.
        if (m_button.getListener() != null) 
        { 
            m_button.getListener().actionPerformed( new ActionEvent(m_button, 
                                                                    ActionEvent.ACTION_PERFORMED, 
                                                                    m_button.getActionCommand() )  );
        }
    } 

    /**
     * Called when the user release the mouse button.
     * @param e Mouse event object
     * @see MouseEvent
     */
    public void mouseReleased(MouseEvent e) 
    { 
        if( m_bAutoRelease == true )
        {
            m_button.setPressed(  false ); 
            m_button.repaint(); 
        }
    }
} 

