 /////////////////////////////////////////////////////////////////////////////
// @doc LegendObj
// @module LegendObj.Java | Implementation of the <c LegendObj> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: LegendObj.java,v 1.1 2000/01/28 20:16:30 daniel Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: LegendObj.java,v $
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

import java.applet.Applet;
import java.awt.Image;
import java.awt.image.PixelGrabber;
import java.awt.event.MouseEvent;                                        
import java.awt.image.ImageObserver;
import java.lang.Integer;
import java.util.Vector;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;

/**
 * This class is used to associated a pixel color and a string. When the user
 * click in the map, it will take the pixel color and prompt the associated string.
 * this object doesn't send any submit. 
 * It's inherit form PointObj.
 * 
 * @author  Guy Parent 
 */
public class LegendObj extends PointObj 
            implements KeyListener
{
    /** String parameter used to get the correlation table */
    final private String kstrLegendParam = "DYN_LEGEND";
    
    /** Use to separate the RGB value */
    protected int  m_alpha  = 0;
    protected int  m_red        = 0;
    protected int  m_green  = 0;
    protected int  m_blue       = 0;

    // Lookup Table attributs.
    protected Vector m_astrDesc = new Vector( 5,5 );
    protected Vector m_anColor = new Vector( 5,5 );

    protected Rosa2000 m_applet;
    
    public LegendObj( Map parent, Rosa2000 applet )
    {
        super( parent );

        m_applet = applet;
        m_applet.addKeyListener( this );

        initLookupTable( applet );
    }

    /** 
     * Get the lookup table from the parameters applet. All item should be separate by "|"
     * token.
     */
    protected void initLookupTable( Rosa2000 applet )
    {
        // Get the parameter string
        String strParamList = applet.getParameter( kstrLegendParam );
        String strItem = new String();
        
        if( strParamList != null )
        {
            int nIndex = 0;
            int nPreviousIndex = 0;
                
            // Parse all string to find the n-1 param
            while( ( nIndex = strParamList.indexOf("|", nIndex++ ) ) != -1 )
            {
                strItem = strParamList.substring( nPreviousIndex, nIndex );
                nIndex++;
                nPreviousIndex = nIndex;
                // Add the new item
                addItemInLookup( strItem );
            }       
            // Take the last param.
            if( nPreviousIndex < strParamList.length()-1 )
            {
                strItem = strParamList.substring( nPreviousIndex );
                addItemInLookup( strItem );
            }
        }
    }

    /**
     * Extract the pixel color value and the associated string and add them
     * in the lookup table.
     * @param strItem Item string. It would define a color and an associated string like
     *               FF0000=Lake And river
     */
    protected void addItemInLookup( String strItem )
    {
        Integer color = null;
        String strDesc = null;
        boolean bError = true;
        
        // Retreive the information from the string
        String strValue = null;
        int nIndex = 0;
        int nPreviousIndex = 0;
        // Find the = key.
        if( ( nIndex = strItem.indexOf("=", nIndex++ ) ) != -1 )
        {
            strValue = strItem.substring( nPreviousIndex, nIndex );
            nIndex++;
            nPreviousIndex = nIndex;
            // Get the color
            try
            {
                int nColorValue = Integer.valueOf( strValue, 16 ).intValue() +0xff000000;
                color = new Integer( nColorValue );
            }
            catch( NumberFormatException e )
            {
                System.err.println( "Invalid color: " + strValue );
                return;
            }

            // Take the last param.
            if( nPreviousIndex < strItem.length()-1 )
                    strDesc = strItem.substring( nPreviousIndex );
                    
            // Add the new Item in the table
            if( color != null && strDesc    != null )
            {
                m_astrDesc.addElement( strDesc );
                m_anColor.addElement( color );
                bError = false;
            }
        
        }   
        
        // Check the error.
        if( bError == true )
        {
            System.err.println( "Invalid item " + strItem + " in the parameter " +  kstrLegendParam );
        }
    }

    /** 
     * Do nothing
     */
    public String getSubmitValue()
    {
        String strRetVal ; // = Integer.toString( x ) + "," + Integer.toString( y );
        return new String();//strRetVal;
    }

    
    private  void  handleSinglePixel( int  pixel)  
    {
        m_alpha =  (pixel  >>  24)  &  0xff;
        m_red       =  (pixel  >>  16)  &  0xff;
        m_green =  (pixel  >>    8)  &  0xff;
        m_blue      =  (pixel            )  &  0xff;
        
        if( Rosa2000.isDebugMode() == true )
            System.out.println( "Alpha = " + m_alpha +" Red = " + m_red + 
                                        " green = " + m_green +" Blue = " + m_blue + " pixel = " + pixel);
    }

    /**
     * Return the color take in the image passed in parameter a the XY location.
     */
    public  int  getPixels(Image  img,  int  x,  int  y,  int  w,  int  h)  
    {
        int[]  pixels  =  new  int[w  *  h];
        
        PixelGrabber  pg  =  new  PixelGrabber(img,  x,  y,  w,  h,  pixels,  0,  w);
        try  
        {
                pg.grabPixels();
        }  
        catch  (InterruptedException  e)  
        {
                System.err.println("interrupted  waiting  for  pixels!");
                return 0;
        }
        if  ((pg.status()  &  ImageObserver.ABORT)  !=  0)  
        {
              System.err.println("image  fetch  aborted  or  errored");
              return 0;
        }
        for  (int  j  =  0;  j  <  h;  j++)  
        {
            for  (int  i  =  0;  i  <  w;  i++)  
            {
                // look at the pixel
//              handleSinglePixel( pixels[j * w + i]);
                return pixels[0];
            }
        }
        return 0;
    }
    
    /////////////////////////////////
    /**
     * Get the pixel color at the location from the mouse event and prompt the associated
     * description.
     */
    public void mouseReleased (MouseEvent evt)
    {
        x = evt.getPoint().x;
        y = evt.getPoint().y;

        // Get the associated description.
        int nColorPixel = getPixels( m_map.getImage(), x, y, 1, 1 );
        String strDesc = getLookupString( new Integer( nColorPixel ) );
    
        // Prompt the description
        if( strDesc != null )
            m_applet.showText( strDesc );
    }

    /**
     * Get the description associated to the color passed in parameter.
     * @param color Color to search for.
     * @return Description associated to the color passed in parameter.
     */
    protected String getLookupString( Integer color )
    {
        int nIndex = m_anColor.indexOf( color );
        
        if( nIndex != -1 )
            return (String)( m_astrDesc.elementAt( nIndex ) );
        
        return null;
    }

    /**
     * Remove the mouse listener and the key listener.
     */
    public void  destroyEvent()
    {
            m_map.removeMouseListener( this );
            m_applet.addKeyListener( this );
    }

    /**
     * Use to stop the processing, If the user press on escape key, the current
     * command will stop.
     */
    public void keyPressed( KeyEvent e ) 
    {
        // Remove the event
        if( e.getKeyCode() == KeyEvent.VK_ESCAPE ) 
        {
            destroyEvent();
            m_map.commitOperation();
        }
        
    }
    /**
     * do nothing
     */
    public void keyReleased( KeyEvent e ) 
    {
    }
    /**
     * do nothing
     */
    public void keyTyped( KeyEvent e ) 
    {
    }
    

}
