/////////////////////////////////////////////////////////////////////////////
// @doc RectButton
// @module RectButton.Java | Implementation of the <c RectButton> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: RectButton.java,v 1.3 2002/08/08 15:37:56 sacha Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: RectButton.java,v $
// Revision 1.3  2002/08/08 15:37:56  sacha
// Put back toolba button when printing but not for Netscape. I still have a problem in netscape. So I just removed the toolbar in Netscape when printing.
//
// Revision 1.2  2000/02/04 15:02:33  gparent
// Added doccumentation.
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

import java.applet.Applet;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Image;

/**
 * This class is used to manage a rectangle button. This control need only
 * one image. It simulates the 3D pressed button.  A ButtonHandler is created
 * to receive the button event.
 * 
 * @author Guy Parent  
 */
public class RectButton extends ButtonCtrl
{

    /**
     * Constructor.
     * @param strName Button name. This string will be used to load all paramerter from the applet.
     * @param img It's the image on the button.
     * @param bAutoPress <b>true</b> is the button will be automaticaly unpress after the action.
     * @param applet Used to draw tooltips.
     */
    public RectButton(String strName,  Image img, boolean bAutoPress,  Rosa2000 applet ) 
    { 
        super( strName, bAutoPress, applet );
        m_Img = img;
        init();
        
        if( Rosa2000.isDebugMode() == true )
            System.out.println("RectButton Constructor");

    } 

    public RectButton(String strName,  Image img, int nXSize, int nYSize, boolean bAutoPress,  Rosa2000 applet ) 
    { 
        super( strName, nXSize, nYSize, bAutoPress, applet );
        m_Img = img;
        init();
    
      if( Rosa2000.isDebugMode() == true )
            System.out.println("RectButton Constructor");
    }
    
    /**
     * Initialize the current button and set the size.  The button grow by 10 pixel
     * to allow the border to draw.
     */
    protected void init( )
    {
        // Set the size of the button
     if( m_nXSize == 0 )
            m_nXSize = m_Img.getWidth( m_applet ) + 10; 
        if( m_nYSize == 0 )
            m_nYSize = m_Img.getHeight( m_applet ) + 10; 
        
        super.init();
    }

    /**
     * Paint the current button.
     */
    public void paint(Graphics g) 
    { 
        if (m_bPressed) 
        { 
            g.setColor(Color.black); 
            g.drawRect(0,0,m_nXSize-1,m_nYSize-1); 
            g.setColor(new Color(120,120,120)); 
            g.drawLine(1,1,m_nXSize-2,1); 
            g.drawLine(1,1,1,m_nYSize-2); 
            g.setColor(new Color(160,160,160)); 
            g.drawLine(2,2,m_nXSize-3,2); 
            g.drawLine(2,2,2,m_nYSize-3); 
            g.setColor(new Color(200,200,200)); 
            g.drawLine(2,m_nYSize-2,m_nXSize-2,m_nYSize-2); 
            g.drawLine(m_nXSize-2,2,m_nXSize-2,m_nYSize-2); 
            g.setColor(new Color(240,240,240)); 
            g.drawLine(3,m_nYSize-3,m_nXSize-3,m_nYSize-3); 
            g.drawLine(m_nXSize-3,3,m_nXSize-3,m_nYSize-3); 
            g.drawImage(m_Img,5,5,m_nXSize-8,m_nYSize-8,this); 
        }
        else 
        { 
            g.setColor(Color.black); 
            g.drawRect(0,0,m_nXSize-1,m_nYSize-1); 
            g.setColor(new Color(200,200,200)); 
            g.drawLine(1,1,m_nXSize-3,1); 
            g.drawLine(1,1,1,m_nYSize-3); 
            g.setColor(new Color(240,240,240)); 
            g.drawLine(2,2,m_nXSize-4,2); 
            g.drawLine(2,2,2,m_nYSize-4); 
            g.setColor(new Color(120,120,120)); 
            g.drawLine(1,m_nYSize-2,m_nXSize-2,m_nYSize-2); 
            g.drawLine(m_nXSize-2,1,m_nXSize-2,m_nYSize-2); 
            g.setColor(new Color(160,160,160)); 
            g.drawLine(2,m_nYSize-3,m_nXSize-3,m_nYSize-3); 
            g.drawLine(m_nXSize-3,2,m_nXSize-3,m_nYSize-3); 
            g.drawImage(m_Img,4,4,m_nXSize-8,m_nYSize-8,this);
        }
    } 


    /**
     * Print the current button.
     */
    public void print(Graphics g) 
    { 
	/* 
	   Netscape print the applet a little bigger than the
	   original component size. This cause netscape to print
	   the applet over some html component. To fix that, we
	   must resize the final result by 75%. This is documented
	   at this place: 

	   http://www.caff.de/workarounds/index.shtml#common_printing

	   If it is not netscape, use a scale factor of 100%
	*/

	double scale=0.75;

	// if not netscape set scale to 100%
	try 
	{
	    Class.forName("netscape.applet.MozillaAppletContext");
	} 
	catch (ClassNotFoundException e) 
	{ 
	    scale = 1.0;
	}

	m_nXSize = (int)(m_nXSize*scale);
	m_nXSize = (int)(m_nYSize*scale);

	paint(g);
    } 
}
