/////////////////////////////////////////////////////////////////////////////
// @doc ToolbarPosition
// @module ToolbarPosition.Java | Implementation of the <c ToolbarPosition> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: ToolbarPanel.java,v 1.8 2002/08/08 15:37:56 sacha Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: ToolbarPanel.java,v $
// Revision 1.8  2002/08/08 15:37:56  sacha
// Put back toolba button when printing but not for Netscape. I still have a problem in netscape. So I just removed the toolbar in Netscape when printing.
//
// Revision 1.7  2001/05/16 18:37:42  assefa
// Clean up tooltip threads.
//
// Revision 1.6  2000/11/03 18:14:51  assefa
// Remove println.
//
// Revision 1.5  2000/10/11 17:08:19  assefa
// Add method getCurrentButton.
//
// Revision 1.4  2000/08/18 14:20:39  assefa
// Add tool tips support.
//
// Revision 1.3  2000/03/30 03:16:41  gparent
// Added the default button.
//
// Revision 1.2  2000/02/04 15:02:33  gparent
// Added doccumentation.
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

import java.awt.Panel;
import java.lang.Exception;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Graphics;

/**
 * Objects of this class are panel that have a row of bitmapped 
 * buttons and optional separators. The buttons can act like push buttons.
 * The toolbar can be locate at the top or bottom in the horizontal orientation
 * or at the left or right side in vertical orientation. 
 * 
 * All parameters are defined in  <a href="http://www.dmsolutions.on.ca/rosa/rosa_params.html">
 * Rosa applet parameters</a>
 * 
 * @author  Guy Parent
 */
public class ToolbarPanel extends Panel
{
    /**  Constant - Location of the toolbar.  Top location.  */
    final public String kstrTop                 = "top";
    /**  Constant - Location of the toolbar.  Bottom location.  */
    final public String kstrBottom              = "bottom";
    /**  Constant - Location of the toolbar.  Left location.  */
    final public String kstrLeft                = "left";
    /**  Constant - Location of the toolbar.  right location.  */
    final public String kstrRight               = "right";

    /** Postfix parameters name using for reading the default image on the button. */
    final protected String kstrDefImageParam    = "_IMG";
    /** Postfix parameters name using for reading the pressed state image on the button. */
    final protected String kstrImagePressParam  = "_IMG_PR";
    
    // Toolbar properties parameters
    /** Alignment of the toolbar. Used in combination of vertical orientation.
     * Top alignment.  */
    final private String kstrTopAling           = "top";
    /** Alignment of the toolbar. Used in combination of vertical orientation.
     * Bottom alignment.  */
    final private String kstrBottomAling        = "bottom";
    /** Alignment of the toolbar. Used in combination of vertical orientation.
     * Middle alignment.  */
    final private String kstrMiddleAling        = "middle";
    /** Alignment of the toolbar. Used in combination of horizontal orientation.
     *  Center alignment.  */
    final private String kstrCenterAling        = "center";
    /** Alignment of the toolbar. Used in combination of horizontal orientation.
     * Left alignment.  */
    final private String kstrLeftAling          = "left";
    /** Alignment of the toolbar. Used in combination of horizontal orientation.
     * Right alignment.  */
    final private String kstrRightAling         = "right";
    
    /** Number of button item in the toolbar  */
    private int     m_nNbrButton     = 0;
    /** TRUE if the orientation of the toolbar is vertical */
    private boolean m_bIsVertical    = false;
    /** Used to define the aligment of the toolbar. */
    private String  m_strAlign       = null;
    /** Used to define the location of the toolbar.  */
    private String  m_strLocation    = null;
    /** this object answer to the button command */
    private Map     m_buttonListener = null;
    
    /** Use to place the buttons */
    private Rosa2000 m_parent        = null;
    
    /** XY location of the toolbar inside the applet.  */
    private Point m_ptLocation;
     
    /**
     * Constructor. Create and initialize the toolbar.
     * @param parent Reference on the applet.
     * @param listener This object will receive the button action.
     * @param strLocation String location.
     * @param strAligment Sting aligment. 
     */
    ToolbarPanel( Rosa2000 parent, Map listener, String strLocation, String strAlignment )
    {
        setVisible( false );

        m_buttonListener = listener;
        m_parent = parent;
        m_ptLocation = new Point(0,0);
            
        setLocation( strLocation ); 

        setAlignment( strAlignment );

        // Remove the layout
        setLayout( null );
        
        // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("ToolbarPanel Constructor param Location =  " + strLocation + "Alignment = " + strAlignment );

    }


    /**
     *  Set the Location of the toolbar.  The string should be define using
     * the constant top, bottom, left or right. Otherwise an error msg will
     * be send to the java console.
     * 
     * @param strLocation Location of the toolbar.
     */ 
    public void setLocation( String strLocation )
    {
        boolean bError = false;
        
        m_strLocation = strLocation;
        
        if( strLocation != null )
        {
            // Check the string on the horizontal orientation.
            if(  strLocation.equals(  kstrTop ) || strLocation.equals(  kstrBottom ) )
            {
                m_bIsVertical = false;
            }
            // Check the string on the vertical orientation.
            else if( strLocation.equals(  kstrLeft ) || strLocation.equals(  kstrRight ) )
            {
                m_bIsVertical = true;
            }
            else
                bError = true;
        }
        else
            bError = true;
        
        // Send an error 
        if( bError )
        {
            System.err.println( "Invalid toolbar Location: " + strLocation );
            m_strLocation = kstrTop; //default
        }
    }

    /**
     * Return the aligment string of the toolbar. It should be top, bottom or middle
     * for the vertical orientation or left, right or center for horizontal horientation.
     */
    public String getAligment()
    {
        return m_strAlign;
    }
    
    /** 
     * Return the string location. It should be top, bottom, left or right.
     */
    public String getTbLocation()
    {
        return m_strLocation;
    }
    
    /**
     * Set the button passed in parameter in pressed state.
     * @param button Button to set.
     */
    public void setButtonPress( ButtonCtrl button )
    {
        for( int i = 0; i <  getComponentCount(); i++ )
        {
            ButtonCtrl buttonItem = (ButtonCtrl)getComponent( i );
            // if the button i == button passed in parameter, setPressed to true.
            buttonItem.setPressed( buttonItem == button );
        }
    }
    
    /**
     * Clean up all the buttons (delete tooltip threads).
     */
    public void cleanUp()
    {
        for( int i = 0; i <  getComponentCount(); i++ )
        {
            ButtonCtrl buttonItem = (ButtonCtrl)getComponent( i );
            if (buttonItem != null)
            {
                //System.out.println("toolbarpanel:cleanUp");
                buttonItem.cleanUp();
            }
        }
    }
    
    /////////////////////////////////////////////////
    /** 
     * Validate and assing the alingment.  The value must be a good combination
     * with the location. top, bottom and middle could be used with left or right
     * and left, right and center could be used with top or bottom.
     */
    public void setAlignment( String strAlignment )
    {
        boolean bError = false;

        // Set the alignment
        m_strAlign   = strAlignment;

        if(  m_bIsVertical )
        {   // Vertical : top, bottom, middle.
            if( (strAlignment != null && 
                 ( strAlignment.equals( kstrTopAling ) || 
                   strAlignment.equals( kstrBottomAling ) || 
                   strAlignment.equals( kstrMiddleAling )  ) ) == false )
            {
                bError = true;
                m_strAlign = kstrTopAling;  // Set to default
            }
        }
        else 
        {   // Horizontal : left, right, center
            if( (strAlignment != null &&  
                 (strAlignment.equals( kstrRightAling ) || 
                  strAlignment.equals( kstrCenterAling ) ||
                  strAlignment.equals( kstrLeftAling ) ) ) == false )
            {
                bError = true;
                m_strAlign = kstrLeftAling; // Set to default
            }
        }
        
        if( bError == true )
            System.err.println( "Invalid toolbar alignment " + strAlignment + ". The default value " +
                                            m_strAlign + " will be assing. " );
    }
    
    
    /**
     * Create a new button using the string name passed in parameter.  If two images
     * are defined to show the button, a round button will be created, otherwise, it
     * will be a rectangle button.
     * 
     * @param strButtonName Name of the button. It use to read the applet parameters.
     */
    public boolean createButton( String strButtonName )
    {
        
        // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("ToolbarPanel::createButton( " + strButtonName + " ) " );

        if( strButtonName.startsWith( SpacingButton.kstrPrafixNameParam) == false )
        {

            // Create the string for the parameters.
            String strImageParam = ButtonCtrl.getPrefixButtonParam() + strButtonName;
            String strImageDef = strImageParam + kstrDefImageParam;
            String strImagePress = strImageParam + kstrImagePressParam;
        
            // Create images on the button
            Image buttonDefImage = readButtonImage( strImageDef );
            Image buttonPressImage = readButtonImage( strImagePress );
        
            if( buttonDefImage != null ) 
            {
                if( buttonPressImage == null )
                {   // Create a button rectangle
                    RectButton button = new RectButton( strButtonName, buttonDefImage, false, m_parent );
                    addButton( button );
                }
                else
                {
                    // Create around rectangle
                    RoundButton button = new RoundButton( strButtonName, buttonDefImage, 
                                                                                    buttonPressImage, false, m_parent );
                    addButton( button );
                }
                    
                return true;
            }
            else 
                return false;   // Button not in the parameters.
        }
        else
        {   // SpacingButton creation.
            SpacingButton button = new SpacingButton( strButtonName, m_bIsVertical );
            addButton( button );
            return true;
        }
            
    }


    /**
     * Read the button image from the parameters applet and load it.
     * 
     * @return Image loaded.
     */
    private Image readButtonImage( String strNameParam )
    {
        // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("ToolbarPanel::readButtonImage( String strNameParam )" );

        // Read the parameter.
        String strDefaultValue = m_parent.getParameter(  strNameParam );
        if( strDefaultValue != null )
     {
         // Load the image.
            MediaTracker tracker = new MediaTracker(this);
            Image buttonImage = m_parent.getImage( m_parent.getDocumentBase(), strDefaultValue );
            tracker.addImage(buttonImage, 0);
            try 
            {
                tracker.waitForID(0);
            } 
            catch (Exception e) 
            {
                System.err.println ("Exception: " + e.getMessage());
                return null;
            }
            if( buttonImage == null )
                System.err.println("ToolbarPanel::readButtonImage error reading image" );
                
                
            return buttonImage;
        }
        
        return null;
    } // end readButtonImage
        

    /**
     * Add the button in the toolbar.
     * 
     * @param button Button to add.
     */
    public void addButton( ButtonCtrl button )
    {
        // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("ButtonCtrl button" );
        
        if( button.isAutoPlacement() == true )
        {
            autoPlace( m_nNbrButton, button );
            m_nNbrButton++;
            
            // add it in the container
            add( button );
        
            // resize the toolbar.
            Dimension tbSize = getCurrentSize();
            setSize( tbSize.width, tbSize.height ); 
        }
        else
            m_parent.add( button );
        
        button.addActionListener( m_buttonListener );

    }

    
    /**
     * Return the dimension of the toolbar.
     */
    public Dimension getCurrentSize() 
    { 
        if( m_bIsVertical == true )
        {   //// Calculate the vertical preferred Size.
            int nXMax   = 0;
            int nYSize = 0;
            
            // For each component get the preferredSize and calculate x and y value.
            Component []aComponent = getComponents();
            for( int i = 0; i < getComponentCount(); i++ )
            { 
                Dimension dim = aComponent[i].getPreferredSize();
                // Calc the max x size
                if( dim.width > nXMax )
                    nXMax = dim.width;
                
                nYSize += dim.height;
            }
                
            return new Dimension( nXMax, nYSize );
        }
        else
        {
            //// Calculate the vertical preferred Size.
            int nYMax   = 0;
            int nXSize = 0;
            
            // For each component get the preferredSize and calculate x and y value.
            Component []aComponent = getComponents();
            for( int i = 0; i < getComponentCount(); i++ )
            { 
                Dimension dim = aComponent[i].getPreferredSize();
                // Calc the max x size
                if( dim.height > nYMax )
                    nYMax = dim.width;
                
                nXSize += dim.width;
            }
                
            return new Dimension( nXSize, nYMax );
        }       
            
    } 

	/**
	 * Return the button associated to the button name passed in parameter.
	 * 
	 * @param strButtonName Button name to find.
	 */
	protected ButtonCtrl getButton( String strButtonName )
	{
		for( int i = 0; i < getComponentCount(); i++ )
		{
			if( getComponent( i ).getClass().getSuperclass().getName().equals( "ButtonCtrl" ) )
			{
				ButtonCtrl currentButton = (ButtonCtrl)getComponent( i );
				if( currentButton.getButtonName() != null &&
					currentButton.getButtonName().equals( strButtonName ) )
					return currentButton;
			}
		}
		return null;
	}

    /**
     * Get the firstbutton which is pressed. If none are pressed null is returned.
     * 
     */
    public ButtonCtrl getCurrentButton()
    {
        for( int i = 0; i <  getComponentCount(); i++ )
        {
            ButtonCtrl buttonItem = (ButtonCtrl)getComponent( i );
            
            if (buttonItem.isPressed())
              return buttonItem;
        }
        return null;
    }

    /**
     * Place automaticaly the button passed in parameter at the index position.
     */
    protected void autoPlace( int nIndex, ButtonCtrl button )
    {
            // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("autoPlace( " + nIndex + " )"  );
        
        if( button.isAutoPlacement() == true )
        {
            ButtonCtrl prevButton = null;
            // Get the previous button;
            if( nIndex > 0  && nIndex <= getComponentCount() )
                prevButton = (ButtonCtrl)getComponent( nIndex - 1 );

            // Get initial position
            Point ptCurLoc = new Point( 0, 0 );

            // Check if it's not the first button
            if( prevButton != null )
            {
                // Find the good position
                if( m_bIsVertical == true )
                {   // vertical
                    ptCurLoc = prevButton.getLocation();
                    ptCurLoc.y += prevButton.getPreferredSize().height;
                }
                else
                {   // horizontal
                    ptCurLoc = prevButton.getLocation();
                    ptCurLoc.x += prevButton.getPreferredSize().width;
                }       
            }

                        // debug
            if( Rosa2000.isDebugMode() == true )
                System.out.println("Button location at: " + ptCurLoc.x + ", " + ptCurLoc.y  );

            // Set the location 
            button.setLocation( ptCurLoc );
        }
    }
        
        
    /**
     * Move the toolbar at the possition passed in parameter.
     */
    public void moveTB( int nX, int nY )
    {
        m_ptLocation = new Point( nX, nY );
        setLocation( m_ptLocation );
    }
    
    /**
     * Place the toolbar at the defined location in the applet.
     */
    public void automaticPlace()
    {
        if( m_bIsVertical == true )
        {
            placeTbVertical();
        }
        else
        {
            placeTbHorizontal();
        }
    }
    
    /**
     * Place the toolbar in vertical orientation
     */
    protected void placeTbVertical()
    {
        // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("ToolbarPanel::placeTbVertical()");

        Dimension mapDim = m_buttonListener.getSize();
        Point  mapLocation  = m_buttonListener.getLocation();
        Dimension tbDim = getSize();
        // Set the y coordinate.
        // Default position
        int nYPosition = mapLocation.y;

        // Set middle
        if( m_strAlign.equals( kstrMiddleAling ) )
            nYPosition += (mapDim.height-tbDim.height)/2;
        else if( m_strAlign.equals( kstrBottomAling ) )
            nYPosition += mapDim.height-tbDim.height;

        int nXPosition = 0;

        if( m_strLocation.equals( kstrLeft ) )
            nXPosition = mapLocation.x - tbDim.width;
        else
            nXPosition = mapLocation.x + mapDim.width;

        if( nXPosition < 0 )
            nXPosition = 0;
        else if( (nXPosition + tbDim.width) > mapDim.width)
            nXPosition = mapDim.width - tbDim.width;
        
        setLocation( nXPosition, nYPosition );      
    }
    
    /**
     * Place the toolbar in horizontal orientation
     */
    protected void placeTbHorizontal()
    {
        // debug
        if( Rosa2000.isDebugMode() == true )
            System.out.println("ToolbarPanel::placeTbHorizontal()");

        Dimension mapDim = m_buttonListener.getSize();
        Point  mapLocation  = m_buttonListener.getLocation();
        Dimension tbDim = getSize();
        // Set the x coordinate.
        // Default position
        int nXPosition = mapLocation.x;

        // Set middle
        if( m_strAlign.equals( kstrCenterAling ) )
            nXPosition += (mapDim.width-tbDim.width)/2;
        else if( m_strAlign.equals( kstrRightAling ) )
            nXPosition += mapDim.width-tbDim.width;

        int nYPosition = 0;

        if( m_strLocation.equals( kstrTop ) )
            nYPosition = mapLocation.y - tbDim.height;
        else
            nYPosition = mapLocation.y + mapDim.height;
        
        if( nYPosition < 0 )
            nYPosition = 0;
        else if( (nYPosition + tbDim.height) > mapDim.height)
            nYPosition = mapDim.height - tbDim.height;
        
        setLocation( nXPosition, nYPosition );      
    }

    public boolean IsVertical()
    {
        return m_bIsVertical;
    }
    
    public boolean IsTop()
    {
        return ( m_strLocation.equals(kstrTop));
    }
    public boolean IsLeft()
    {
        return ( m_strLocation.equals(kstrLeft));
    }

    public void printAll(Graphics g)
    {
	super.printAll(g);
    }
}

